CREATE OR REPLACE PACKAGE mam_email IS
/**
*
* @file mam_email.pck
*
* @copy
*       This is free and unencumbered software released into the public domain.
*
*       Anyone is free to copy, modify, publish, use, compile, sell, or
*       distribute this software, either in source code form or as a compiled
*       binary, for any purpose, commercial or non-commercial, and by any
*       means.
*
*       In jurisdictions that recognize copyright laws, the author or authors
*       of this software dedicate any and all copyright interest in the
*       software to the public domain. We make this dedication for the benefit
*       of the public at large and to the detriment of our heirs and
*       successors. We intend this dedication to be an overt act of
*       relinquishment in perpetuity of all present and future rights to this
*       software under copyright law.
*
*       THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
*       EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*       MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*       IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
*       OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
*       ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
*       OTHER DEALINGS IN THE SOFTWARE.
*
*       For more information, please refer to <http://unlicense.org/>
*
* @desc
*       Send emails with attachments.
*       This solution assumes that you have access to an internal smtp-host
*       If you want to send e.g. via Gmail then search for stunnel or "UTL_SMTP over SSL"
*       # https://hostelfour.wordpress.com/2012/03/29/sending-email-from-oracle-xe-using-gmail/
*       Or use utl_smtp.auth
*       # https://arkatec.wordpress.com/2011/08/15/sending-email-using-oracle-database-and-google-mail-service/
*       # http://oracle.ninja/sending-secure-e-mails-out-of-the-database-ssltls-utl_smtp-openssl-acl-wallet/
*       # https://www.experts-exchange.com/articles/5915/How-to-Authenticate-to-an-SMTP-server-with-Oracle-PL-SQL.html
*
* @hist
*  +Table History
*  +-----------+-------------------+-------+----------------------------
*   When        Who                 Version What
*  +-----------+-------------------+-------+----------------------------
*   23.03.2018  Matzberger Marcus   1       new
*   15.06.2018  Matzberger Marcus   2       add iCalendar event
*   08.03.2019  Matzberger Marcus   3       add inline images
*  +-----------+-------------------+-------+----------------------------
*/

    TYPE rec_mailbody IS RECORD (
         line       VARCHAR2(1000)
        );
    TYPE tbl_mailbody IS
        TABLE OF rec_mailbody
        INDEX BY BINARY_INTEGER;

    -- If contentID is not NULL the BLOB is traeted as Inline picture
    -- and can be referenced in HTML with <img src="cid:contentID">
    TYPE rec_attachment IS RECORD (
         fileName   VARCHAR2(200)
        ,fileCLOB   CLOB
        ,fileBLOB   BLOB
        ,fileICAL   VARCHAR2(2000)
        ,contentID  PLS_INTEGER
        );
    TYPE tbl_attachments IS
        TABLE OF rec_attachment
        INDEX BY BINARY_INTEGER;

    g_attachments   tbl_attachments;
    g_mailbody      tbl_mailbody;

PROCEDURE debug_on;
PROCEDURE debug_off;

/**
* @desc     Set the name of the mail server
*           You can use a hard coded default instead - see package body
* @param    serverName  Name of the server
*/
PROCEDURE setMailserver
    (
     serverName             IN VARCHAR2
    );

/**
* @desc     Send E-Mails. bodyText as VARCHAR2
* @param    recipient:  Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
*                       More than one recipient as comma separated list
* @param    CC          Send copy to
* @param    BCC         Send blind copy to
* @param    sender:     Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>'
* @param    subject:    E-Mail subject. Max 100 characters
* @param    bodyText:   Mail text as array of plain text. tbl_mailbody
* @param    bodyHTML:   Mail text as array of HTML text. tbl_mailbody
* @param    attachments:   Attachments as array: tbl_attachments
*/
PROCEDURE send
    (
     recipient              IN VARCHAR2
    ,cc                     IN VARCHAR2 := NULL
    ,bcc                    IN VARCHAR2 := NULL
    ,sender                 IN VARCHAR2
    ,subject                IN VARCHAR2
    ,bodyText               IN tbl_mailbody
    ,bodyHTML               IN tbl_mailbody    := g_mailbody
    ,attachments            IN tbl_attachments := g_attachments
    );

/**
* @desc     Send E-Mails. bodyText as VARCHAR2
* @param    recipient:  Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
*                       More than one recipient as comma separated list
* @param    CC          Send copy to
* @param    BCC         Send blind copy to
* @param    sender:     Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
* @param    subject:    E-Mail subject. Max 100 characters
* @param    bodyText:   Mail plain text. Max 1000 characters
* @param    bodyHTML:   Mail text as HTML. Max 1000 characters
* @param    attachments:   Attachments as array: tbl_attachments
* @param    iCalendar:  attach calendar event
*/
PROCEDURE send
    (
     recipient              IN VARCHAR2
    ,cc                     IN VARCHAR2 := NULL
    ,bcc                    IN VARCHAR2 := NULL
    ,sender                 IN VARCHAR2
    ,subject                IN VARCHAR2
    ,bodyText               IN VARCHAR2
    ,bodyHTML               IN VARCHAR2 := NULL
    ,attachments            IN tbl_attachments := g_attachments
    );

/**
* @desc     Send E-Mails. bodyText as VARCHAR2
* @param    recipient:  Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
*                       More than one recipient as comma separated list
* @param    sender:     Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
* @param    subject:    E-Mail subject. Max 100 characters
* @param    bodyText:   Mail text. Max 1000 characters
*/
PROCEDURE sendSimple
    (
     recipient              IN VARCHAR2
    ,sender                 IN VARCHAR2
    ,subject                IN VARCHAR2
    ,bodyText               IN VARCHAR2
    );

/**
* @desc     Create iCalendar event
*           https://tools.ietf.org/html/rfc5546
* @param    sender:     Valid E-Mail address myself@example.com or with alias 'My name <myself@example.com>
* @param    startDate:  Begin date time
* @param    endDate:    End date time
* @param    subject:    Event subject. Max 100 characters
* @param    description: Short description. Max 1000 characters
* @param    vLocation:  Location like "Conference Room 2"
* @param    alarm:      Alarm in minutes before startDate
* @return iCalendar event as string
*/
FUNCTION createEvent
    (
     sender                 IN VARCHAR2
    ,startDate              IN DATE
    ,endDate                IN DATE
    ,subject                IN VARCHAR2
    ,description            IN VARCHAR2 := NULL
    ,vLocation              IN VARCHAR2 := NULL
    ,vAlarm                 IN PLS_INTEGER := NULL
    )
    RETURN VARCHAR2;

END mam_email;
/
SHOW ERRORS